from pylab import *
rc('font',   size=12)

def load_data(fname):
    return loadtxt(fname, dtype={
        'names'   : ('size', 'omp', 'mpi', 'setup', 'solve', 'iters'),
        'formats' : ('i8', 'i4', 'i4', 'f8', 'f8', 'i4')
        })

def set_ticks(ax, t):
    ax.set_xticks(t)
    ax.get_xaxis().set_major_formatter(matplotlib.ticker.ScalarFormatter())
    ax.get_xaxis().set_tick_params(which='minor', size=0)
    ax.get_xaxis().set_tick_params(which='minor', width=0)

fig,gs = subplots(4, 2, figsize=(8,10))
handles = []
omps = set()

for k,fname in enumerate(('dmem_data/mn4/linear_strong.dat', 'dmem_data/mn4/const_strong.dat')):
    data = load_data(fname)
    for omp in sorted(unique(data['omp'])):
        omps.add(omp)

        d = data[data['omp'] == omp]
        c = unique(d['mpi'] * omp)
        m = unique(d['mpi'])

        setup = array([min(d[d['mpi']==i]['setup']) for i in m])
        solve = array([min(d[d['mpi']==i]['solve']) for i in m])
        iters = array([min(d[d['mpi']==i]['iters']) for i in m])
        total = setup + solve

        ax = gs[0,k]
        h = ax.loglog(c, total, '.-')
        ax.set_ylim([1e0, 2e2])
        set_ticks(ax, c)
        if k == 0: handles.append(h[0])
        ideal = total[0] * c[0] / c
        if omp == 12:
            hi = ax.plot(c,ideal,'k:', zorder=1, linewidth=1, alpha=0.5)

        ax = gs[1,k]
        ax.loglog(c, setup, '.-')
        ax.set_ylim([1e-1, 1e2])
        ideal = setup[0] * c[0] / c
        if omp == 12:
            ax.plot(c,ideal,'k:', zorder=1, linewidth=1, alpha=0.5)
        set_ticks(ax, c)

        ax = gs[2,k]
        ax.loglog(c, solve, '.-')
        ideal = solve[0] * c[0] / c
        if omp == 12:
            ax.plot(c,ideal,'k:', zorder=1, linewidth=1, alpha=0.5)
        ax.set_ylim([1e-1, 2e2])
        set_ticks(ax, c)

        ax = gs[3,k]
        ax.semilogx(c, iters, '.-')
        ax.set_ylim([0,300])
        set_ticks(ax, c)

    gs[3,k].set_xlabel('Number of cores (MPI * OMP)')

for fname in ('dmem_data/mn4/trilinos_strong.dat', 'dmem_data/mn4/trilinos_strong_ddml.dat'):
    tri = loadtxt(fname, dtype={
        'names'   : ('mpi', 'size', 'iters', 'setup', 'solve'),
        'formats' : ('i4', 'i8', 'i4', 'f8', 'f8')
        })

    for k in (0,1):
        h = gs[0,k].plot(tri['mpi'], tri['setup'] + tri['solve'], '.-')
        if k == 0: handles += h

        gs[1,k].plot(tri['mpi'], tri['setup'], '.-')
        gs[2,k].plot(tri['mpi'], tri['solve'], '.-')
        gs[3,k].plot(tri['mpi'], tri['iters'], '.-')

gs[0,0].set_title('Linear deflation')
gs[0,0].set_ylabel('Total time')
gs[0,1].set_title('Constant deflation')
gs[1,0].set_ylabel('Setup time')
gs[2,0].set_ylabel('Solve time')
gs[3,0].set_ylabel('Iterations')

figlegend(handles + hi, ['OMP={}'.format(i) for i in sorted(omps)]
        + ['Trilinos/ML', 'Trilinos/DD-ML', 'Ideal scaling'],
       ncol=3, loc='lower center')

fig.tight_layout()
fig.suptitle('Strong scaling of the Poisson problem on the MareNostrum 4 cluster')
fig.subplots_adjust(top=0.93, bottom=0.12)

show()